unit MainForm;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, LResources, Forms, Controls, Graphics, Dialogs, ExtCtrls,
  StdCtrls;

resourcestring
  rsTheSpringHas = 'The spring has broken (T_T)';

type

  { TMain }

  TMain = class(TForm)
    Edit1: TEdit;
    Edit2: TEdit;
    Edit3: TEdit;
    Label1: TLabel;
    Label2: TLabel;
    Label3: TLabel;
    Memo1: TMemo;
    PaintBox: TPaintBox;
    Timer1: TTimer;
    Timer2: TTimer;
    procedure Edit2Change(Sender: TObject);
    procedure Edit3Change(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure PaintBoxMouseDown(Sender: TOBject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure PaintBoxMouseMove(Sender: TObject; Shift: TShiftState; X,
      Y: Integer);
    procedure PaintBoxMouseUp(Sender: TOBject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure Timer1Timer(Sender: TObject);
    procedure Timer2Timer(Sender: TObject);
  private
    procedure GetParameterValues;
    procedure DrawSpring(l: Double);
  public
    { public declarations }
  end; 

var
  Main: TMain;
  ox, a, v, y_, r, F, m, g, k, ls, mag, dt: Double;
  
implementation

{ TMain }

procedure TMain.FormCreate(Sender: TObject);
begin
  ox := 50;
  v := 0;
  r := 0.01;
  g := 9.81;
  ls := 0.08;
  mag := 2720;
  Top := 0;
  Left := 0;
end;

procedure TMain.Edit2Change(Sender: TObject);
begin
  Timer2.Enabled := true;
end;

procedure TMain.Edit3Change(Sender: TObject);
begin

end;

procedure TMain.PaintBoxMouseDown(Sender: TOBject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
begin
  Timer1.Enabled := false;
end;

procedure TMain.PaintBoxMouseMove(Sender: TObject; Shift: TShiftState; X,
  Y: Integer);
begin
  if (ssLeft in Shift) then
  begin
    y_ := - r + Y / mag;
    PaintBox.Invalidate;
    Application.ProcessMessages;
    DrawSpring(y_);
  end;
end;

procedure TMain.PaintBoxMouseUp(Sender: TOBject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
begin
  GetParameterValues;
  dt := 1.1 * Timer1.Interval / 1000.0;
  Timer1.Enabled := true;
end;

procedure TMain.Timer1Timer(Sender: TObject);
begin
  a := m * g - k * (y_ - ls) - F * v;
  v := v + a * dt;
  y_ := y_ + v * dt;
  PaintBox.Invalidate;
  Application.ProcessMessages;
  DrawSpring(y_);
end;

procedure TMain.Timer2Timer(Sender: TObject);
begin
  GetParameterValues();
  y_ := ls + m * g / k;
  PaintBox.Invalidate;
  Application.ProcessMessages;
  DrawSpring(y_);
  Timer2.Enabled := false;
end;

procedure TMain.GetParameterValues;
begin
  m := StrToFloat(Edit1.Text);
  if (Edit2.Text <> '') and (Edit2.Text <> '0') then
    k := StrToFloat(Edit2.Text);
  F := StrToFloat(Edit3.Text);
end;

procedure TMain.DrawSpring(l: Double);
var
  i: Integer;
  d: Double;
  Points: array[0..21] of TPoint;
begin
  points[0].x := Trunc(ox);
  points[0].y := 0;
  d := 40 * 40 - (l * mag / 20.0) * (l * mag / 20.0);
  if d > 0 then
    d := sqrt(d)
  else
  begin
    ShowMessage(rsTheSpringHas);
    Timer1.Enabled := false;
    Timer2.Enabled := false;
    Exit;
  end;
  for i := 1 to 20 do
  begin
    points[i].x := Trunc(ox + d * (i - 2 * Trunc(i / 2)) - d / 2.0);
    points[i].y := points[0].y + Trunc(l * mag / 20.0 * (i - 0.5));
  end;
  points[21].x := Trunc(ox);
  points[21].y := Trunc(l * mag);
  PaintBox.Canvas.Pen.Style := psDot;
  PaintBox.Canvas.MoveTo(Trunc(ox - 50), Trunc((ls + m * g / k + r) * mag));
  PaintBox.Canvas.LineTo(Trunc(ox + 50), Trunc((ls + m * g / k + r) * mag));
  PaintBox.Canvas.Pen.Color := clBlue;
  PaintBox.Canvas.MoveTo(Trunc(ox - 50), Trunc((ls + r) * mag));
  PaintBox.Canvas.LineTo(Trunc(ox + 50), Trunc((ls + r) * mag));
  PaintBox.Canvas.Pen.Style := psSolid;
  PaintBox.Canvas.Pen.Color := clBlack;
  PaintBox.Canvas.Polyline(Points);
  PaintBox.Canvas.Brush.Color := clBlue;
  PaintBox.Canvas.Ellipse(Trunc(ox - r * mag), points[21].y, Trunc(ox + r * mag), points[21].y + Trunc(r * 2 * mag));
end;

initialization
  {$I MainForm.lrs}

end.

